#ifndef LINT
static char *rcsid="$Id: clamav.c 295 2005-05-13 14:58:27Z crosser $";
#endif

/*
	WHAT IS IT:
		modularized contentfilter for Zmailer
	COPYRIGHT:
		(c) 2003-2005 Eugene G. Crosser <crosser@average.org>
	LICENSE:
		The same set as apply to Zmailer code
*/

#include "config.h"

#include <sys/types.h>
#ifdef HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#ifdef STDC_HEADERS
# include <stdio.h>
# include <stdlib.h>
# include <string.h>
#endif
#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif
#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif

#include <zmscanner.h>
#include <clamav.h>

#include "clamav_config.h"
#include "report.h"

int clamav_refreshdb(clamav_cfg_t *);
void pre_scan(void);
void post_scan(void);

#define CONFFILE "clamav.conf"

#define min(x,y) ((x<y)?x:y)

static int
clamav_setup(void **priv)
{
	clamav_cfg_t *cfg;
	int rc;
	char fbuf[128];

	DPRINT(("clamav_setup starting\n"));

	snprintf(fbuf,sizeof(fbuf),"%s/%s",modconfdir(),CONFFILE);
	cfg=clamav_config(fbuf);
	cfg->finaldbdir=cfg->dbdir?cfg->dbdir:cl_retdbdir();
	(*priv)=(void *)cfg;
	rc=clamav_refreshdb(cfg);
	if (rc) return rc;
	ERRLOG((LOG_INFO,"clamav: use %s scanning",
					cfg->filescan?"file":"memory"));
	ERRLOG((LOG_INFO,"clamav: %s (%s) OK",VERSION,rcsid));

	return 0;
}

static void
clamav_term(void **priv)
{
	clamav_cfg_t *cfg=*((clamav_cfg_t **)priv);

	if (cfg) {
		cl_free(cfg->root);
		free(cfg);
	}
	*priv=NULL;
}

static int
clamav_scan(char *stage,int depth,slab_t data,varpool_t vp,void *priv)
{
	clamav_cfg_t *cfg=(clamav_cfg_t *)priv;
	int rc=-1;
	const char *virname;
	mattr_t *atr=data.atr;
	char ans[128];

	DPRINT(("clamav_scan enter\n"));

	if (!cfg->scantext &&
	    !((atr) && (slab_size(atr->content_type)) &&
	      (strncasecmp(atr->content_type.beg,"text",
				min(4,slab_size(atr->content_type))) != 0))) {
		DPRINT(("not scanning text type\n"));
		return ZMSCAN_CONTINUE;
	}

	if (cfg->filescan) {
		FILE *tmpf=tmpfile();
		size_t rest=slab_size(data);
		char *p=data.beg;

		DPRINT(("clamav file scan, options=0x%04x\n",cfg->clamopts));
		if (tmpf == NULL) {
			ERRLOG((LOG_ERR,"clamav_scan: open tmp file: %m"));
			goto bailout;
		}
		while (rest) {
			size_t wrote=fwrite(p,1,rest,tmpf);
			rest-=wrote;
			if (ferror(tmpf)) {
				ERRLOG((LOG_ERR,"clamav_scan: fwrite tmp: %m"));
				goto bailout;
			}
		}
		if (fflush(tmpf)) {
			ERRLOG((LOG_ERR,"clamav_scan: fflush tmp: %m"));
			goto bailout;
		}
		pre_scan();
		rc=cl_scandesc(fileno(tmpf),&virname,NULL,cfg->root,
			&cfg->limits,cfg->clamopts);
		post_scan();
		bailout:
		fclose(tmpf);
	} else {
		pre_scan();
		rc=cl_scanbuff(data.beg,slab_size(data),&virname,cfg->root);
		post_scan();
	}
	switch (rc) {
	case 0:
		DPRINT(("clamav scan result clean\n"));
		return ZMSCAN_CONTINUE;
	case 1:
		ERRLOG((LOG_INFO,"clamav: blocked %s",virname));
		snprintf(ans,sizeof(ans),
			"-1 550 5.7.0 Found virus %s in the message",virname);
		vp_set_str(vp,VP_ANSWER_STR,ans);
		return ZMSCAN_STOP;
	default:
		ERRLOG((LOG_ERR,"clamav_scan result rc=%d\n",rc));
		return ZMSCAN_CONTINUE;
	}
}

ZMS_MODULE("content","clamav",clamav_setup,clamav_term,clamav_scan);

